class HmiKeyConstants:
    APU_IP_ADDRESS = "apu_ip_address"
    HMI_HOSTNAME = "hmi_hostname"
    HMI_HOMEPAGE = "hmi_homepage"
    HMI_IP_ADDRESS_TYPE = "hmi_ip_address_type"
    HMI_IP_ADDRESS = "hmi_ip_address"
    HMI_NETMASK = "hmi_netmask"
    HMI_DEFAULT_GATEWAY = "hmi_default_gateway"
    HMI_DNS = "hmi_dns"
    HMI_LAN_TYPE = "hmi_lan_type"
    HMI_WIRELESS_SSID = "hmi_wireless_ssid"
    HMI_WIRELESS_PASSPHRASE = "hmi_wireless_passphrase"
    HMI_MACHINE_TYPE = "hmi_machine_type"


class HmiMachineTypeConstants:
    RPI4_HMI = "rpi4hmi"
    ALL_IN_ONE_HMI = "allinonehmi"
